#ifndef _RB_H_
#define _RB_H_

// trieda pre spravu parametrov RigidBody **********************************
class CRigidBody
{
 private:
  // hmotnost RB - mass
  // lepsie je pouzit inverznu (prevratenu) hodnotu
  // povinne nastavit
  float M_inv;

  // tenzory zotrvacnosti RB v body a world space - inertia tensors
  // lepsie je pouzit inverzne (prevratene) hodnoty
  // povinne nastavit
  mat3x3 Ibs_inv,I_inv;

  // poloha RB vo world space - position - center of mass
  // povinne nastavit
  vec3 s;
  // rotacna matica RB vo world space - rotation matrix
  CQuaternion R;

  // sila a krutiaci moment posobiace na RB - force and torque
  vec3 F,T;
  // zrychlenie, uhlove zrychlenie RB - acceleration, angular acceleration
  vec3 a,aa;
  // casovy prirastok rychlosti RB
  vec3 delta_v;
  // priamociara a uhlova rychlost RB - linear and angular speed
  vec3 v,w;

  // utlm priamociarej a uhlovej rychlosti - linear and angular damping
  // povinne nastavit
  float l_damping,a_damping;
  // koeficient trenia
  float friction;

  // transformacna matica pre vzajomny prevod suradnic medzi body a world space
  // vzpocitava sa z s a R
  mat4x3 t_mat;

  // kineticka energia RB
  float energy,act_energy,ratio;
  // energia RB, pri ktorej RB zmeni stav na neaktivny
  float inactive_energy;
  // RB je/nie je aktivne
  bool is_active;
  // RB moze/nemoze byt neaktivne
  bool can_be_inactive;

  // vrcholy RB a trojuholniky, z ktorych je zlozene RB
  vec3 vrts[8];
  vector<triangle> tris;

  // pomocna premenna
  float t[9];
 public:
  // konstruktor
  CRigidBody() {}

  // rozmery RB
  float sx,sy,sz;

  // pocet trojuholnikov, z ktorych je zlozene RB
  int tris_num;

  // hmotnost RB
  void set_M(const float A) { if (A>0.0f) M_inv=1.0f/A; }
  void set_M_inv(const float A) { M_inv=A; }
  float get_M_inv(void) const { return(M_inv); }

  // tenzor zotrvacnosti RB v body space
  void set_Ibs(const mat3x3 A) { Ibs_inv.set_inverse(A); }
  void set_Ibs_inv(const mat3x3 A) { Ibs_inv=A; }
  void get_Ibs_inv(mat3x3 *A) const { *A=Ibs_inv; }
  mat3x3 get_Ibs_inv(void) const { return(Ibs_inv); }
  // tenzor zotrvacnosti RB vo world space
  void get_I_inv(mat3x3 *A) const { *A=I_inv; }
  mat3x3 get_I_inv(void) const { return(I_inv); }

  // poloha RB
  void set_s(const vec3 A) { s=A; }
  void set_s(const float A,const float B,const float C) { s.x=A; s.y=B; s.z=C; }
  void add_s(const vec3 A) { s+=A; }
  void get_s(vec3 *A) const { *A=s; }
  vec3 get_s(void) const { return(s); }

  // rotacna matica RB - reprezentovana quaternionom
  void set_R(const CQuaternion A) { R=A; R.normalize(); }
  void set_R(const float A,const float B,const float C,const float D) {
   R.r=A; R.i=B; R.j=C; R.k=D; R.normalize();
  }
  void add_R(const vec3 A) {
   CQuaternion q;
   get_R(&q); q.add_vector(A); set_R(q);
  }
  void get_R(CQuaternion *A) const { *A=R; }
  CQuaternion get_R(void) const { return(R); }

  // sila a krutiaci moment posobiace na RB
  void clear_F_T(void) { F.clear(); T.clear(); }
  // RB sa aktivuje
  void add_F(const vec3 A) { F+=A; set_is_active(); }
  // RB sa aktivuje
  void add_T(const vec3 A,const vec3 P) { F+=A; T+=cross(P-s,A); set_is_active(); }
   // RB sa aktivuje
  void add_T(const vec3 A) { T+=A; set_is_active(); }

  // zrychlenie a predchadzajuce zrychlenie RB
  void clear_a_aa(void) { a.clear(); aa.clear(); }
  void set_a(const vec3 A) { a=A; }
  void set_a(const float A,const float B,const float C) { a.x=A; a.y=B; a.z=C; }
  void get_a(vec3 *A) const { *A=a; }
  vec3 get_a(void) const { return(a); }

  // casovy prirastok rychlosti RB
  vec3 get_delta_v(void) const { return(delta_v); }
  // priamociara a uhlova rychlost RB
  void set_v(const vec3 A) { v=A; }
  void set_v(const float A,const float B,const float C) { v.x=A; v.y=B; v.z=C; }
  vec3 get_v(void) const { return(v); }
  void add_v(const vec3 A) { v+=A; }
  void set_w(const vec3 A) { w=A; }
  void set_w(const float A,const float B,const float C) { w.x=A; w.y=B; w.z=C; }
  vec3 get_w(void) const { return(w); }
  void add_w(const vec3 A) { w+=A; }
  // impulz v += J / M   w += I_inv . ( (B-s) x J )
  // RB sa aktivuje
  void add_J(const vec3 A,const vec3 B) { v+=A*M_inv; w+=I_inv*cross(B-s,A); set_is_active(); }

  // utlm priamociarej a uhlovej rychlosti
  void set_damping(const float A,const float B) { l_damping=A; a_damping=B; }
  void set_l_damping(const float A) { l_damping=A; }
  float get_l_damping(void) const { return(l_damping); }
  void set_a_damping(const float A) { a_damping=A; }
  float get_a_damping(void) const { return(a_damping); }
  // koeficient trenia (dynamicke)
  void set_friction(const float A) { friction=A; }
  float get_friction(void) const { return(friction); }

  // transformacna matica pre vzajomny prevod suradnic medzi body a world space
  void get_t_mat(mat4x3 *A) const { memcpy(A,t_mat.d,sizeof(mat4x3)); }
  void get_t_mat(float A[16]) const {
   memcpy(A,t_mat.d,sizeof(float)*12);
   A[12]=A[13]=A[14]=0.0f; A[15]=1.0f;
  }
  mat4x3 get_t_mat(void) const { return(t_mat); }
  vec3 get_t_mat_column(const unsigned int A) const { return(t_mat.get_column_vector(A)); }
  // vektor (stlpec) z transformacnej matice
  vec3 get_vec(const unsigned int A) const { return(get_t_mat_column(A)); }
  vec3 get_rotX(void) const { return(get_vec(0)); }
  vec3 get_rotY(void) const { return(get_vec(1)); }
  vec3 get_rotZ(void) const { return(get_vec(2)); }
  vec3 get_pos(void) const { return(get_vec(3)); }

  // uprava transformacnej matice pre pouzitie v OpenGL
  // stlpce OpenGL matice = riadky transformacnej matice
  void get_GLtransform(float A[16]) const {
   A[0]=t_mat.d[0]; A[4]=t_mat.d[1]; A[8] =t_mat.d[2];  A[12]=t_mat.d[3];
   A[1]=t_mat.d[4]; A[5]=t_mat.d[5]; A[9] =t_mat.d[6];  A[13]=t_mat.d[7];
   A[2]=t_mat.d[8]; A[6]=t_mat.d[9]; A[10]=t_mat.d[10]; A[14]=t_mat.d[11];
   A[3]=0.0f;       A[7]=0.0f;       A[11]=0.0f;        A[15]=1.0f;
  }

  // prevod suradnic z world do body space
  vec3 vertex_to_bs(const vec3 A) const { return(t_mat.mul_transp(A)); }
  // prevod suradnic z body do world space
  vec3 vertex_to_ws(const vec3 A) const { return(t_mat.mul(A)); }

  // energia RB, pri ktorej RB zmeni stav na neaktivny
  void set_inactive_energy(const float A) { inactive_energy=A; }
  float get_inactive_energy(void) const { return(inactive_energy); }
  // RB je/nie je aktivne
  void set_is_active(const bool A=true) {
   if (A) { is_active=true; energy=2.0f*inactive_energy; }
   else { is_active=false; v.clear(); w.clear(); }
  }
  bool get_is_active(void) const { return(is_active); }
  // RB moze/nemoze byt neaktivne
  void set_can_be_inactive(const bool A=true) {
   can_be_inactive=A;
   // ak je RB neaktivne a pritom nemoze byt neaktivne, aktivuje sa
   if ((!can_be_inactive) && (!is_active)) set_is_active();
  }
  bool get_can_be_inactive(void) const { return(can_be_inactive); }

  // vrcholy RB a trojuholniky, z ktorych je zlozene RB
  vec3 get_vrts(const int A) { return(vrts[A]); }
  triangle *get_tri(const int A) { return(&tris[A]); }

  // inicializacia RB
  // hS - polovica rozmerov RB
  void rb_init(vec3 hS);

  // normalizacia R, vypocet t_mat, I_inv - vzdy po zmene R, alebo Ibs
  void update_params(void);

  // vypocet casoveho kroku fyzikalnej simulacie
  // pouzije sa Newton-Eulerova metoda - linearna aproximacia hodnot integralu
  // pomocou prvej derivacie
  void integrate(float DT);
};

#endif
