// hlavickove subory *******************************************************
#include "../includes.h"
// vlastny .h subor
#include "RBcollisions.h"

// CCollision --------------------------------------------------------------

// vypocet col_t, col_n_t, r, dv, v0, numer ********************************
void CCollision::calc_params(float DT)
{
 // ak neexistuje RBa
 if (!RBa) return;

 // tangenty v koliznom bode
 if (fabs(col_n.x)>fabs(col_n.y)) {
  d=1.0f/sqrt(col_n.z*col_n.z+col_n.x*col_n.x);
  // po zanedbani y
  col_t[0].x=col_n.z*d;
  col_t[0].y=0.0f;
  col_t[0].z=-col_n.x*d;
  // col_t[1] = col_n x col_t[0]
  col_t[1].x=col_n.y*col_t[0].x;
  col_t[1].y=col_n.z*col_t[0].x-col_n.x*col_t[0].z;
  col_t[1].z=-col_n.y*col_t[0].x;
 }
 else {
  d=1.0f/sqrt(col_n.z*col_n.z+col_n.y*col_n.y);
  // po zanedbani x
  col_t[0].x=0.0f;
  col_t[0].y=-col_n.z*d;
  col_t[0].z=col_n.y*d;
  // col_t[1] = col_n x col_t[0]
  col_t[1].x=col_n.y*col_t[0].z-col_n.z*col_t[0].y;
  col_t[1].y=-col_n.x*col_t[0].z;
  col_t[1].z=col_n.x*col_t[0].y;
 }
 // matica pre ulozenie col_n, col_t
 //    col_n.x    col_n.y    col_n.z
 // col_t[0].x col_t[0].y col_t[0].z
 // col_t[1].x col_t[1].y col_t[1].z
 col_n_t.set_components(col_n,col_t[0],col_t[1]);

 // vzdialenost kolizneho bodu od stredu RBa a RBb
 r[0]=col_p-RBa->get_s();
 if (RBb) r[1]=col_p-RBb->get_s();

 // relativna rychlost RBa pred aplikovanim impulzu
 // vpred_a = vpred_a + wpred_a x r_a
 h0=RBa->get_v()+RBa->get_w().cross(r[0]);
 // v0 = n . vpred_a
 h0=col_n_t.mul_T(h0);

 // prirastok rychlosti
 dv=col_n_t.mul_T(RBa->get_delta_v());
 // odstranenie zlozky rychlosti v smere normaly, zostanu iba tangentne zlozky,
 // ktore budu neskor zrusene aplikovanim trenia
 dv.x=0.0;
 h0+=dv;

 // v0 = n . ( vpred_a + wpred_a x r_a )
 v0=h0;

 if (RBb) {
  // relativna rychlost RBb pred aplikovanim impulzu
  // vpred_b = vpred_b + w x r_b
  h1=RBb->get_v()+RBb->get_w().cross(r[1]);
  h1=col_n_t.mul_T(h1);

  // prirastok rychlosti
  dv=col_n_t.mul_T(RBb->get_delta_v());
  // odstranenie zlozky rychlosti v smere normaly, zostanu iba tangentne zlozky,
  // ktore budu neskor zrusene aplikovanim trenia
  dv.x=0.0;
  h1+=dv;

  // v0 = n . ( vpred_a + wpred_a x r_a ) - n . ( vpred_b + wpred_b x r_b )
  v0-=h1;
 }

 // vypocet delenca (numerator) pre vypocet j
 calc_numer(DT);
}

// vypocet delenca (numerator) pre vypocet j - zavisly na v0 ***************
void CCollision::calc_numer(float DT)
{
 // prirastok rychlosti - redukcia vibracii
 dv.x=0.0;
 if (RBa->get_is_active()) dv.x=(RBa->get_delta_v()).dot(col_n);
 if (RBb && RBb->get_is_active()) dv.x-=(RBb->get_delta_v()).dot(col_n);

 // numerator = -(1 + COR) * v0 = - v0 - COR * v0
 if (fabs(v0.x)<0.25f) numer=-v0.x; // COR = 0.0f - redukcia vibracii
 else numer=-v0.x-cor*(v0.x-dv.x);
}

// vypocet velkosti impulzu so zanedbanim trenia ***************************
vec3 CCollision::calc_j_frictionless(mat3x3 Iia,mat3x3 Iib)
{
 // h0 = Iia * ( r_a x n ) x r_a
 h0=r[0].cross(col_n); h0=Iia.mul(h0); h0=h0.cross(r[0]);
 // denom = 1/M_a + n . h0
 denom=RBa->get_M_inv()+h0.dot(col_n);

 if (RBb) {
  // h1 = Iib * ( r_b x n ) ) x r_b
  h1=r[1].cross(col_n); h1=Iib.mul(h1); h1=h1.cross(r[1]);
  // denom += 1/M_b + n . h1
  denom+=RBb->get_M_inv()+h1.dot(col_n);
 }

 // h0.x = j = numer / ( 1/M_a + n . ( ( Iia * ( r_a x n ) ) x r_a ) +
 //                      1/M_b + n . ( ( Iib * ( r_b x n ) ) x r_b ) )
 h0.set(numer/denom,0.0f,0.0f);

 return(h0);
}

// vypocet velkosti impulzu s aplikaciou trenia ****************************
vec3 CCollision::calc_j_friction(mat3x3 Iia,mat3x3 Iib)
{
 // trenie = sila, nie impulz, friction = force, not impulse
 // staticke - 2 objekty sa vzajomne nepohybuju, sila posobiaca na 1 z nich
 //            nie je schopna tento objekt posunut (oddelit)
 // dynamicke - po prekonani sily, ktora je pricinou statickeho trenia
 //             dostatocne velkou externou silou sa 1 z objektov zacne pohybovat
 //             pricom nan posobi dynamicke trenie
 // izotropicke/anizotropicke - ma v roznych smeroch rovnake/rozne koeficienty trenia
 //
 // aplikacia trenia vo forme impulzu:
 // J = delta v * M, F = J * delta t = delta v * M * delta t
 // reakcna sila F = rychlost, ktoru pozadujeme zrusit * hmotnost * casovy interval
 // Jp = delta Jn * u
 // Jp - impulz posobiaci v koliznej rovine
 // delta Jn - impulz posobiaci v smere koliznej normaly
 // u - koeficient trenia
 //
 // pouzijeme rovnaky vztah ako pri vypocte j_frictionless avsak vo vsetkych 3
 // rovinach:
 // h0 = 1/M_a + n . ( ( Iia * ( r_a x n ) ) x r_a )
 // namiesto n (1 rozmer) pouzijeme col_n_t (3 rozmery):
 // h0 = 1/M_a + col_n_t . ( ( Iia * ( r_a x col_n_t ) ) x r_a )
 // po vyuziti nasledujucich pravidiel:
 //  vektor1 x vektor2 = skew-symmetric matica z vektora1 * vektor2
 //  vektor2 x vektor1 = - skew-symmetric matica z vektora1 * vektor2
 //  ssm inverzna = ssm transponovana
 //  matica col_n_t je pre obe RB v kolizii rovnaka
 // vztah pre vypocet h0 mozno upravit:
 // h0 = 1/M_a + col_n_t . ( ( -ssm ) * ( Iia * ( ssm * col_n_t ) ) )

 // d = 1/M_a
 d=RBa->get_M_inv();
 // ( -ssm ) * ( Iia * ( ssm ) )
 ssm.set_skew_symetric(r[0]);
 mat0=Iia*ssm; mat0=(-1.0f*ssm)*mat0;

 if (RBb) {
  // d += 1/M_b
  d+=RBb->get_M_inv();
  // ( -ssm ) * ( Iib * ( ssm ) )
  ssm.set_skew_symetric(r[1]);
  mat1=Iib*ssm; mat1=(-1.0f*ssm)*mat1;

  mat0+=mat1;
 }

 // mat1 - matica pre transformaciu impulzu na rychlost vo world space
 // mat1 = col_n_t . ( ( -ssm ) * ( Iia * ( ssm ) ) + ( -ssm ) * ( Iib * ( ssm ) ) ) * col_n_t
 mat1=col_n_t.transpose()*mat0; mat1*=col_n_t;
 // mat1 - matica pre transformaciu impulzu na rychlost v body space
 //                              skalar 0 0
 // vektor1 * skalar = vektor1 * 0 skalar 0
 //                              0 0 skalar
 // kombinacia priamociarej a uhlovej rychlosti mat1 0 4 8 += ( 1/M_a + 1/M_b )
 mat1.d[0]+=d; mat1.d[4]+=d; mat1.d[8]+=d;

 // impulz v jednotkach rychlosti
 jm=mat1.inverse();
 // rychlosti, ktore je potrebne zrusit
 h0.set(numer,-v0.y,-v0.z);

 // impulz potrebny na zrusenie rychlosti
 // staticke trenie
 h0=jm.mul(h0);

 // velkost impulzu posobiaceho v koliznej rovine
 d=sqrt(h0.y*h0.y+h0.z*h0.z);
 // ak d > delta Jn * u - pouzije sa dynamicke trenie
 if (d>(h0.x*RBa->get_friction())) {
  // normalizacia y a z zloziek Jp
  h0.y/=d; h0.z/=d;
  h0.x=mat1.d[0]+mat1.d[1]*RBa->get_friction()*h0.y+
                 mat1.d[2]*RBa->get_friction()*h0.z;
  h0.x=numer/h0.x;
  // pozadovane velkosti y a z zloziek Jp
  h0.y*=RBa->get_friction()*h0.x;
  h0.z*=RBa->get_friction()*h0.x;
 }

 return(h0);
}

// CCollisionDetection -----------------------------------------------------

// detekcia kolizii medzi RB a koliznymi rovinami **************************
bool CCollisionDetection::box_plane(const CCollisionBox &B,const CCollisionPlane &P)
{
 // ak nie je mozne detekovat dalsiu koliziu
 if (col_remain<=0) return(false);

 // vzdialenost stredu RB od koliznej roviny
 d=P.normal.dot(B.rb->get_pos())+P.dist;
 // ak je vzdialenost stredu RB od koliznej roviny vacsia, alebo rovna
 // ako vzdialenost stredu RB od jeho povrchu, nenastala kolizia
 if (d>=center_to_surface(B,P.normal)) return(false);
 // pre vsetky vrcholy RB
 loopi(8) {
  // vrchol RB vo world space
  pv=B.rb->vertex_to_ws(B.rb->get_vrts(i));
  // vzdialenost vrcholu RB od koliznej roviny
  d=pv.dot(P.normal)+P.dist;
  // ak lezi vrchol za koliznou rovinou, nastala kolizia
  if (d<0.0f) {
   // pridanie parametrov detekovanej kolizie do pola
   add_collision(B.rb,NULL,pv,P.normal,-d,0.2f);
   // ak nie je mozne detekovat dalsiu koliziu
   if (col_remain<=0) return(true);
  }
 }
 return(true);
}

// test vzdialenosti stredu b2 (b1) od roviny prechadzajucej stredom b1 (b2)
bool CCollisionDetection::check_plane(vec3 N,const unsigned int I)
{
 if (N.squaredlen()<0.0001f) return(true);
 N.normalize();
 // vzdialenost stredu b2 (b1) od roviny prechadzajucej stredom b1 (b2)
 d=fabs(centerv.dot(N));
 // odpocitanie vzdialenosti stredu b2 (b1) od roviny prechadzajucej stredom b1 (b2)
 // od vzdialenosti stredov RB od ich povrchov v smere vektora N
 d=center_to_surface(b1,N)+center_to_surface(b2,N)-d;
 // ak je d <= 0.0f, nedoslo ku kolizii
 if (d<=0.0f) return(false);
 // ak je d <= p_min, nastavia sa nove p_min a i_min
 if (d<p_min) { p_min=d; i_min=I; }

 return(true);
}

// detekcia kolizii medzi RB a ostatnymi RB ********************************
bool CCollisionDetection::box_box(const CCollisionBox &B1,const CCollisionBox &B2)
{
 // ak nie je mozne detekovat dalsiu koliziu
 if (col_remain<=0) return(false);
 // RB, pre ktore je detekovana kolizia
 b1=B1; b2=B2;
 // vektor stred b1 -> stred b2
 centerv=b2.rb->get_pos()-b1.rb->get_pos();

 p_min=9999999.0f; // minimalna hlbka prieniku b1 do b2
 i_min=6;          // index roviny s minimalnym prienikom b1 do b2

 // test vzdialenosti stredu b2 (b1) od roviny prechadzajucej stredom b1 (b2)
 // get_rot - normala roviny
 // p_min - minimalna hlbka prieniku b1 do b2
 // i_min - index roviny s minimalnym prienikom b1 do b2
 // testuju sa 3 roviny prechadzajuce stredom b1
 if (!check_plane(b1.rb->get_rotX(),0)) return(false);
 if (!check_plane(b1.rb->get_rotY(),1)) return(false);
 if (!check_plane(b1.rb->get_rotZ(),2)) return(false);
 // testuju sa 3 roviny prechadzajuce stredom b2
 if (!check_plane(b2.rb->get_rotX(),3)) return(false);
 if (!check_plane(b2.rb->get_rotY(),4)) return(false);
 if (!check_plane(b2.rb->get_rotZ(),5)) return(false);

 // ak b1 kolidovalo s b2
 if (i_min<3) {
  // normala koliznej roviny
  n=b1.rb->get_vec(i_min); if (n.dot(centerv)>0.0f) n.invert();
  // vrchol b2
  pv=b2.halfSize;
  if (b2.rb->get_rotX().dot(n)<0.0f) pv.x=-pv.x;
  if (b2.rb->get_rotY().dot(n)<0.0f) pv.y=-pv.y;
  if (b2.rb->get_rotZ().dot(n)<0.0f) pv.z=-pv.z;
  // pridanie parametrov detekovanej kolizie do pola
  // b1 kolidovalo s b2, konkretne s jeho vrcholom pv
  add_collision(b1.rb,b2.rb,b2.rb->vertex_to_ws(pv),n,p_min,0.2f);

  return(true);
 } // ak b2 kolidovalo s b1
 else if (i_min<6) {
       i_min-=3;
       // normala koliznej roviny
       n=b2.rb->get_vec(i_min); if (n.dot(-1.0f*centerv)>0.0f) n.invert();
       // vrchol b1
       pv=b1.halfSize;
       if (b1.rb->get_rotX().dot(n)<0.0f) pv.x=-pv.x;
       if (b1.rb->get_rotY().dot(n)<0.0f) pv.y=-pv.y;
       if (b1.rb->get_rotZ().dot(n)<0.0f) pv.z=-pv.z;
       // pridanie parametrov detekovanej kolizie do pola
       // b2 kolidovalo s b1, konkretne s jeho vrcholom pv
       add_collision(b2.rb,b1.rb,b1.rb->vertex_to_ws(pv),n,p_min,0.2f);

       return(true);
      }

 return(false);
}

// zmena poloh a rotacnych matic RBa a RBb aplikovanim impulzu *************
void CCollisionDetection::change_s_R(CCollision *C,vec3 Ds[2],vec3 DR[2])
{
 denom=0.0f;

 // Ii = I_inv
 C->RBa->get_I_inv(&Ii[0]);
 if (C->RBb) C->RBb->get_I_inv(&Ii[1]);

 // h0 = Iia * ( r_a x n ) x r_a
 h0=C->r[0].cross(C->col_n); h0=Ii[0].mul(h0); h0=h0.cross(C->r[0]);
 // u[0] = n . ( ( Iia * ( r_a x n ) ) x r_a )
 u[0]=h0.dot(C->col_n);
 // denom += 1/M_a + n . ( ( Iia * ( r_a x n ) ) x r_a )
 denom+=C->RBa->get_M_inv()+u[0];

 if (C->RBb) {
  // h1 = Iib * ( r_b x n ) x r_b
  h1=C->r[1].cross(C->col_n); h1=Ii[1].mul(h1); h1=h1.cross(C->r[1]);
  // u[1] = n . ( ( Iib * ( r_b x n ) ) x r_b )
  u[1]=h1.dot(C->col_n);
  // denom += 1/M_b + n . ( ( Iib * ( r_b x n ) ) x r_b )
  denom+=C->RBb->get_M_inv()+u[1];
 }

 // priamociara zlozka pohybu = hlbka prieniku * 1/M_a / celkova zotrvacnost
 // uhlova zlozka pohybu = hlbka prieniku * uhlova zotrvacnost RBa / celkova zotrvacnost
 sl[0]=C->col_penetr*(C->RBa->get_M_inv()/denom); sa[0]=C->col_penetr*(u[0]/denom);

 // limit uhloveho pohybu ak je hmotnost velka a hodnota denom mala
 // uprava zavisi na velkosti RBa pricom sa vyuzije hodnota r_a
 h0=C->r[0]; h0+=C->col_n*(-C->r[0].dot(C->col_n));
 d=h0.magnitude()*0.2f;
 if (sa[0]<-d) { sla=sl[0]+sa[0]; sa[0]=-d; sl[0]=sla-sa[0]; }
 else if (sa[0]>d) { sla=sl[0]+sa[0]; sa[0]=d; sl[0]=sla-sa[0]; }

 // bez uhloveho pohybu = bez rotacie
 if (sa[0]==0.0f) DR[0].clear();
 else {
  // Iia * ( r_a x n ) - impulzny krutiaci moment potrebny pre zmenu 1 jednotky pohybu
  // Rj = Iia * ( r_a x n ) / uhlova zotrvacnost RBa - rotacia pre ziskanie 1 jednotky pohybu
  // R = Rj * uhlova zlozka pohybu - celkova potrebna hodnota rotacie
  // R = ( Iia * ( r_a x n ) ) / uhlova zotrvacnost RBa * uhlova zlozka pohybu
  h0=C->r[0].cross(C->col_n); DR[0]=Ii[0].mul(h0)/u[0]*sa[0];
 }

 // zmena polohy RBa = kolizna normala * priamociara zlozka pohybu
 Ds[0]=C->col_n*sl[0];
 C->RBa->add_s(Ds[0]);

 // zmena rotacnej matice RBa
 C->RBa->add_R(DR[0]);

 // normalizacia R, vypocet t_mat, I_inv - vzdy po zmene R, alebo Ibs
 C->RBa->update_params();

 if (C->RBb) {
  // priamociara zlozka pohybu = -hlbka prieniku * 1/M_b / celkova zotrvacnost
  // uhlova zlozka pohybu = -hlbka prieniku * uhlova zotrvacnost RBb / celkova zotrvacnost
  sl[1]=-C->col_penetr*(C->RBb->get_M_inv()/denom); sa[1]=-C->col_penetr*(u[1]/denom);

  // limit uhloveho pohybu ak je hmotnost velka a hodnota denom mala
  // uprava zavisi na velkosti RBb pricom sa vyuzije hodnota r_b
  h1=C->r[1]; h1+=C->col_n*(-C->r[1].dot(C->col_n));
  d=h1.magnitude()*0.2f;
  if (sa[1]<-d) { sla=sl[1]+sa[1]; sa[1]=-d; sl[1]=sla-sa[1]; }
  else if (sa[1]>d) { sla=sl[1]+sa[1]; sa[1]=d; sl[1]=sla-sa[1]; }

  // bez uhloveho pohybu = bez rotacie
  if (sa[1]==0.0f) DR[1].clear();
  else {
   // Iib * ( r_b x n ) - impulzny krutiaci moment potrebny pre zmenu 1 jednotky pohybu
   // Rj = Iib * ( r_b x n ) / uhlova zotrvacnost RBb - rotacia pre ziskanie 1 jednotky pohybu
   // R = Rj * uhlova zlozka pohybu - celkova potrebna hodnota rotacie
   // R = ( Iib * ( r_b x n ) ) / uhlova zotrvacnost RBb * uhlova zlozka pohybu
   h1=C->r[1].cross(C->col_n); DR[1]=Ii[1].mul(h1)/u[1]*sa[1];
  }

  // zmena polohy RBb = kolizna normala * priamociara zlozka pohybu
  Ds[1]=C->col_n*sl[1];
  C->RBb->add_s(Ds[1]);

  // zmena rotacnej matice RBb
  C->RBb->add_R(DR[1]);

  // normalizacia R, vypocet t_mat, I_inv - vzdy po zmene R, alebo Ibs
  C->RBb->update_params();
 }
}

// zmena rychlosti RBa a RBb aplikovanim impulzu ***************************
void CCollisionDetection::change_v_w(CCollision *C,vec3 Dv[2],vec3 Dw[2])
{
 // Ii = I_inv
 C->RBa->get_I_inv(&Ii[0]);
 if (C->RBb) C->RBb->get_I_inv(&Ii[1]);

 // vypocet velkosti impulzu
 if (C->RBa->get_friction()==0.0f) j=C->calc_j_frictionless(Ii[0],Ii[1]);
 else j=C->calc_j_friction(Ii[0],Ii[1]);

 // vypocet impulzu
 // J = j * n
 J=C->col_n_t.mul(j);
 // torqueJ = r_a x J
 torqueJ=C->r[0].cross(J);
 // zmena uhlovej rychlosti delta w = Iia * torqueJ
 Dw[0]=Ii[0].mul(torqueJ);
 // zmena priamociarej rychlosti delta v = J / M_a
 Dv[0].clear(); Dv[0]+=J*(C->RBa->get_M_inv());

 // vpo_a = vpred_a + J / M_a
 C->RBa->add_v(Dv[0]);
 // wpo_a = wpred_a + Iia * ( r_a x J )
 C->RBa->add_w(Dw[0]);

 if (C->RBb) {
  // na RBb posobi opacny impulz
  J.invert();
  // torqueJ = r_b x J
  torqueJ=C->r[1].cross(J);
  // zmena uhlovej rychlosti delta w = Iib * torqueJ
  Dw[1]=Ii[1].mul(torqueJ);
  // zmena priamociarej rychlosti delta v = J / M_b
  Dv[1].clear(); Dv[1]+=J*(C->RBb->get_M_inv());

  // vpo_b = vpred_b + J / M_b
  C->RBb->add_v(Dv[1]);
  // wpo_b = wpred_b + Iib * ( r_b x J )
  C->RBb->add_w(Dw[1]);
 }
}

// ak je jedno RB v kolizii aktivne a druhe neaktivne, neaktive sa aktivuje
void CCollisionDetection::activate_inactive_RB(void)
{
 if (c[id].RBb) {
  // XOR: 0 ^ 0 = 0, 1 ^ 1 = 0, 0 ^ 1 = 1, 1 ^ 0 = 1
  if (c[id].RBa->get_is_active() ^ c[id].RBb->get_is_active()) {
   if (c[id].RBa->get_is_active()) c[id].RBb->set_is_active();
   else c[id].RBa->set_is_active();
  }
 }
}

// detekcia a riesenie kolizii *********************************************
void CCollisionDetection::detect_and_resolve_collisions(const CCollisionBox *B,const unsigned int BN,
                                                        const CCollisionPlane *P,const unsigned int PN,
                                                        float DT)
{
 // detekcia kolizii -------------------------------------------------------
 // reset pola obsahujuceho parametre detekovanych kolizii
 col_reset();

 // pre vsetky RB
 loopi(BN) {
  // detekcia kolizii medzi RB a koliznymi rovinami
  loopj(PN) box_plane(B[i],P[j]);
  // detekcia kolizii medzi RB a ostatnymi RB
  for (k=i+1;k<BN;k++) box_box(B[i],B[k]);
 }

 // riesenie kolizii -------------------------------------------------------
 // ak nebola detekovana kolizia
 if (col_num==0) return;

 // pole obsahujuce parametre detekovanych kolizii
 c=col_first;
 // pre vsetky detekovane kolizie
 loopi(col_num) {
  // vypocet col_t, col_n_t, r, dv, v0, numer
  c[i].calc_params(DT);
 }

 // zmena poloh a rotacnych matic RBa a RBb aplikovanim impulzu ------------
 iter_used=0;
 while (iter_used<s_iter_num) {
  // najdenie kolizie, pre ktoru je nutne vykonat maximalnu upravy polohy
  id=col_num; max=s_epsilon;
  loopi(col_num) { if (c[i].col_penetr>max) { max=c[i].col_penetr; id=i; } }
  // ak nebola najdena ziadna kolizia s pozadovanou minimalnou zmenou polohy
  if (id==col_num) break;

  // ak je jedno RB v kolizii aktivne a druhe neaktivne, neaktive sa aktivuje
  activate_inactive_RB();

  // zmena poloh a rotacnych matic RBa a RBb aplikovanim impulzu
  change_s_R(&c[id],d_s,d_r);

  // tym RB, ktore kolidovali s RBa alebo RBb sa nastavi nova hodnota
  // col_penetr
  loopi(col_num) {
   // ak 1. RB v kolizii = 1. RB, ktoremu boli zmenene s a R
   if (c[i].RBa==c[id].RBa) {
    d_sr=d_s[0]+d_r[0].cross(c[i].r[0]); c[i].col_penetr-=d_sr.dot(c[i].col_n);
   }
   // ak 1. RB v kolizii = 2. RB, ktoremu boli zmenene s a R
   if (c[i].RBa==c[id].RBb) {
    d_sr=d_s[1]+d_r[1].cross(c[i].r[0]); c[i].col_penetr-=d_sr.dot(c[i].col_n);
   }
   if (c[i].RBb) {
    // ak 2. RB v kolizii = 1. RB, ktoremu boli zmenene s a R
    if (c[i].RBb==c[id].RBa) {
     d_sr=d_s[0]+d_r[0].cross(c[i].r[1]); c[i].col_penetr+=d_sr.dot(c[i].col_n);
    }
    // ak 2. RB v kolizii = 2. RB, ktoremu boli zmenene s a R
    if (c[i].RBb==c[id].RBb) {
     d_sr=d_s[1]+d_r[1].cross(c[i].r[1]); c[i].col_penetr+=d_sr.dot(c[i].col_n);
    }
   }
  }
  iter_used++;
 }

 // zmena rychlosti RBa a RBb aplikovanim impulzu --------------------------
 iter_used=0;
 while (iter_used<v_iter_num) {
  // najdenie kolizie, pre ktoru je nutne vykonat maximalnu upravy rychlosti
  id=col_num; max=v_epsilon;
  loopi(col_num) { if (c[i].numer>max) { max=c[i].numer; id=i; } }
  // ak nebola najdena ziadna kolizia s pozadovanou minimalnou zmenou rychlosti
  if (id==col_num) break;

  // ak je jedno RB v kolizii aktivne a druhe neaktivne, neaktive sa aktivuje
  activate_inactive_RB();

  // zmena rychlosti RBa a RBb aplikovanim impulzu
  change_v_w(&c[id],d_v,d_w);

  // tym RB, ktore kolidovali s RBa alebo RBb sa nastavia nove hodnoty
  // v0 a numer
  loopi(col_num) {
   // ak 1. RB v kolizii = 1. RB, ktoremu boli zmenene v a w
   if (c[i].RBa==c[id].RBa) {
    d_vw=d_v[0]+d_w[0].cross(c[i].r[0]);
    c[i].v0+=c[i].col_n_t.mul_T(d_vw); c[i].calc_numer(DT);
   }
   // ak 1. RB v kolizii = 2. RB, ktoremu boli zmenene v a w
   if (c[i].RBa==c[id].RBb) {
    d_vw=d_v[1]+d_w[1].cross(c[i].r[0]);
    c[i].v0+=c[i].col_n_t.mul_T(d_vw); c[i].calc_numer(DT);
   }
   if (c[i].RBb) {
    // ak 2. RB v kolizii = 1. RB, ktoremu boli zmenene v a w
    if (c[i].RBb==c[id].RBa) {
     d_vw=d_v[0]+d_w[0].cross(c[i].r[1]);
     c[i].v0-=c[i].col_n_t.mul_T(d_vw); c[i].calc_numer(DT);
    }
    // ak 2. RB v kolizii = 2. RB, ktoremu boli zmenene v a w
    if (c[i].RBb==c[id].RBb) {
     d_vw=d_v[1]+d_w[1].cross(c[i].r[1]);
     c[i].v0-=c[i].col_n_t.mul_T(d_vw); c[i].calc_numer(DT);
    }
   }
  }
  iter_used++;
 }
}
